<?php

namespace Drupal\php_compatibility_scanner\Batch;

/**
 * Batch operations for PHP compatibility scanning.
 */
class ScanBatch {

  /**
   * Create batch for scanning modules.
   *
   * @param array $modules
   *   Array of module names to scan.
   * @param string $target_version
   *   Target PHP version.
   *
   * @return array
   *   Batch definition.
   */
  public static function createBatch(array $modules, $target_version) {
    $operations = [];

    // Initialize operation
    $operations[] = [
      '\Drupal\php_compatibility_scanner\Batch\ScanBatch::initialize',
      [$target_version],
    ];

    // Add operation for each module
    foreach ($modules as $module_name) {
      $operations[] = [
        '\Drupal\php_compatibility_scanner\Batch\ScanBatch::scanModule',
        [$module_name, $target_version],
      ];
    }

    // Finish operation
    $operations[] = [
      '\Drupal\php_compatibility_scanner\Batch\ScanBatch::finalize',
      [],
    ];

    return [
      'title' => t('Scanning @count modules for PHP @version compatibility', [
        '@count' => count($modules),
        '@version' => $target_version,
      ]),
      'operations' => $operations,
      'finished' => '\Drupal\php_compatibility_scanner\Batch\ScanBatch::finished',
      'init_message' => t('Starting compatibility scan...'),
      'progress_message' => t('Processed @current out of @total modules.'),
      'error_message' => t('The scan has encountered an error.'),
    ];
  }

  /**
   * Initialize batch.
   */
  public static function initialize($target_version, &$context) {
    $context['results']['target_version'] = $target_version;
    $context['results']['current_version'] = \Drupal::service('php_compatibility_scanner.scanner')->getCurrentPhpVersion();
    $context['results']['modules'] = [];
    $context['results']['start_time'] = microtime(TRUE);

    $context['message'] = t('Initializing scan...');
  }

  /**
   * Scan a single module.
   */
  public static function scanModule($module_name, $target_version, &$context) {
    $scanner = \Drupal::service('php_compatibility_scanner.scanner');
    $module_discovery = \Drupal::service('php_compatibility_scanner.module_discovery');

    // Get module info
    $module_type = $module_discovery->getModuleType($module_name);
    $module_path = $module_discovery->getModulePath($module_name);

    // Scan the module
    $result = $scanner->scanModule($module_name, $target_version);

    // Store results
    $context['results']['modules'][$module_name] = [
      'name' => $module_name,
      'type' => $module_type,
      'path' => $module_path,
      'issues' => $result['issues'],
      'success' => $result['success'],
      'error' => $result['error'] ?? NULL,
    ];

    // Update message
    $issue_count = count($result['issues']);
    $context['message'] = t('Scanned @module: @count issues found', [
      '@module' => $module_name,
      '@count' => $issue_count,
    ]);
  }

  /**
   * Finalize batch.
   */
  public static function finalize(&$context) {
    // Calculate total time
    $context['results']['duration'] = microtime(TRUE) - $context['results']['start_time'];

    // Save results
    $scanner = \Drupal::service('php_compatibility_scanner.scanner');
    $scanner->saveScanResults($context['results']);

    $context['message'] = t('Finalizing results...');
  }

  /**
   * Batch finished callback.
   */
  public static function finished($success, $results, $operations) {
    if ($success) {
      // Count total issues
      $total_issues = 0;
      $total_errors = 0;
      $total_warnings = 0;

      foreach ($results['modules'] as $module) {
        foreach ($module['issues'] as $issue) {
          $total_issues++;
          if ($issue['severity'] === 'error') {
            $total_errors++;
          } else {
            $total_warnings++;
          }
        }
      }

      $message = t('Scan completed in @time seconds. Found @total issues (@errors errors, @warnings warnings).', [
        '@time' => round($results['duration'], 2),
        '@total' => $total_issues,
        '@errors' => $total_errors,
        '@warnings' => $total_warnings,
      ]);

      \Drupal::messenger()->addStatus($message);
    }
    else {
      \Drupal::messenger()->addError(t('The scan did not complete successfully.'));
    }
  }
}