<?php

namespace Drupal\php_compatibility_scanner\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\OpenModalDialogCommand;
use Drupal\Core\Render\Markup;
use Drupal\php_compatibility_scanner\Service\ScannerService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Controller for displaying PHP compatibility issues.
 */
class IssuesController extends ControllerBase {

  /**
   * The scanner service.
   *
   * @var \Drupal\php_compatibility_scanner\Service\ScannerService
   */
  protected $scannerService;

  /**
   * Constructs a new IssuesController object.
   *
   * @param \Drupal\php_compatibility_scanner\Service\ScannerService $scanner_service
   *   The scanner service.
   */
  public function __construct(ScannerService $scanner_service) {
    $this->scannerService = $scanner_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('php_compatibility_scanner.scanner')
    );
  }

  /**
   * Displays issues for a module in a modal dialog.
   *
   * @param string $module_name
   *   The name of the module.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse|\Drupal\Core\Render\MarkupInterface|array
   *   The response.
   */
  public function moduleIssues($module_name, Request $request) {
    // Get scan results.
    $results = $this->scannerService->getScanResults();

    // Check if module exists in results.
    if (empty($results['modules'][$module_name])) {
      $message = $this->t('No scan results available for this module.');

      // If AJAX request, return as dialog
      if ($request->headers->get('X-Requested-With') === 'XMLHttpRequest') {
        $response = new AjaxResponse();
        $response->addCommand(new OpenModalDialogCommand(
          $this->t('Error'),
          $message,
          ['width' => '400', 'dialogClass' => 'php-compatibility-issues-modal']
        ));
        return $response;
      }

      return ['#markup' => $message];
    }

    $module_data = $results['modules'][$module_name];
    $current_version = $results['current_version'] ?? $this->scannerService->getCurrentPhpVersion();
    $target_version = $results['target_version'] ?? $this->scannerService->getNextPhpVersion();

    // Build the content.
    $build = [
      '#theme' => 'php_compatibility_scanner_issues',
      '#module_name' => $module_name,
      '#module_type' => $module_data['type'],
      '#module_path' => $module_data['path'] ?? '',
      '#issues' => $module_data['issues'] ?? [],
      '#current_version' => $current_version,
      '#target_version' => $target_version,
      '#attached' => [
        'library' => ['php_compatibility_scanner/php_compatibility_scanner.styles'],
      ],
    ];

    // Get the rendered HTML
    $renderer = \Drupal::service('renderer');
    $html = $renderer->render($build);

    // If this is an AJAX request, return an AJAX response
    if ($request->headers->get('X-Requested-With') === 'XMLHttpRequest') {
      $response = new AjaxResponse();

      // Count issues by severity.
      $errors = 0;
      $warnings = 0;

      foreach ($module_data['issues'] as $issue) {
        if ($issue['severity'] === 'error') {
          $errors++;
        }
        elseif ($issue['severity'] === 'warning') {
          $warnings++;
        }
      }

      // Create modal title based on results.
      $title = $this->t('@module: @count PHP Compatibility Issues', [
        '@module' => $module_name,
        '@count' => count($module_data['issues']),
      ]);

      if ($errors > 0 || $warnings > 0) {
        $title = $this->t('@module: @errors Errors, @warnings Warnings', [
          '@module' => $module_name,
          '@errors' => $errors,
          '@warnings' => $warnings,
        ]);
      }

      // Return the modal dialog command.
      $response->addCommand(new OpenModalDialogCommand(
        $title,
        Markup::create($html),
        [
          'width' => '800',
          'dialogClass' => 'php-compatibility-issues-modal',
        ]
      ));

      return $response;
    }

    // Return the render array for non-AJAX requests.
    return $build;
  }

}
