<?php

namespace Drupal\php_compatibility_scanner\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\php_compatibility_scanner\Service\ModuleDiscoveryService;
use Drupal\php_compatibility_scanner\Service\ScannerService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form for running the PHP compatibility scan.
 */
class ScanForm extends FormBase {

  /**
   * The module discovery service.
   *
   * @var \Drupal\php_compatibility_scanner\Service\ModuleDiscoveryService
   */
  protected $moduleDiscovery;

  /**
   * The scanner service.
   *
   * @var \Drupal\php_compatibility_scanner\Service\ScannerService
   */
  protected $scannerService;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * Constructs a new ScanForm object.
   *
   * @param \Drupal\php_compatibility_scanner\Service\ModuleDiscoveryService $module_discovery
   *   The module discovery service.
   * @param \Drupal\php_compatibility_scanner\Service\ScannerService $scanner_service
   *   The scanner service.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   */
  public function __construct(
    ModuleDiscoveryService $module_discovery,
    ScannerService $scanner_service,
    DateFormatterInterface $date_formatter
  ) {
    $this->moduleDiscovery = $module_discovery;
    $this->scannerService = $scanner_service;
    $this->dateFormatter = $date_formatter;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('php_compatibility_scanner.module_discovery'),
      $container->get('php_compatibility_scanner.scanner'),
      $container->get('date.formatter')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'php_compatibility_scanner_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Add container for better styling.
    $form['#prefix'] = '<div class="php-compatibility-scanner-container">';
    $form['#suffix'] = '</div>';

    // Get PHP version information.
    $current_version = $this->scannerService->getCurrentPhpVersion();
    $target_version = $this->scannerService->getNextPhpVersion();

    // Header info section.
    $form['info'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['php-compatibility-scanner-info'],
      ],
    ];

    $form['info']['heading'] = [
      '#type' => 'html_tag',
      '#tag' => 'h2',
      '#value' => $this->t('PHP Compatibility Scanner'),
    ];

    $form['info']['description'] = [
      '#type' => 'html_tag',
      '#tag' => 'p',
      '#value' => $this->t('This tool scans your installed modules to detect compatibility issues with newer PHP versions. It helps you prepare for PHP upgrades by identifying deprecated or removed features that need updating.'),
    ];

    $form['info']['versions'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['php-version-info'],
      ],
    ];

    $form['info']['versions']['current'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['current-version'],
      ],
      'label' => [
        '#type' => 'html_tag',
        '#tag' => 'div',
        '#value' => $this->t('Current PHP Version'),
        '#attributes' => [
          'class' => ['version-label'],
        ],
      ],
      'value' => [
        '#type' => 'html_tag',
        '#tag' => 'div',
        '#value' => $current_version,
        '#attributes' => [
          'class' => ['version-number'],
        ],
      ],
    ];

    $form['info']['versions']['target'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['target-version'],
      ],
      'label' => [
        '#type' => 'html_tag',
        '#tag' => 'div',
        '#value' => $this->t('Target PHP Version'),
        '#attributes' => [
          'class' => ['version-label'],
        ],
      ],
      'value' => [
        '#type' => 'html_tag',
        '#tag' => 'div',
        '#value' => $target_version,
        '#attributes' => [
          'class' => ['version-number'],
        ],
      ],
      'description' => [
        '#type' => 'html_tag',
        '#tag' => 'div',
        '#value' => $this->t('The scan will check compatibility with this PHP version.'),
        '#attributes' => [
          'class' => ['version-description'],
        ],
      ],
    ];

    // Last scan information if available.
    $last_scan_time = $this->scannerService->getLastScanTime();
    if ($last_scan_time) {
      $form['info']['last_scan'] = [
        '#type' => 'html_tag',
        '#tag' => 'div',
        '#value' => $this->t('Last scan: @date', [
          '@date' => $this->dateFormatter->format($last_scan_time, 'medium'),
        ]),
        '#attributes' => [
          'class' => ['last-scan-info'],
        ],
      ];
    }

    // Scan options section.
    $form['scan_options'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['scan-options'],
      ],
    ];

    $form['scan_options']['heading'] = [
      '#type' => 'html_tag',
      '#tag' => 'div',
      '#value' => $this->t('What would you like to scan?'),
      '#attributes' => [
        'class' => ['scan-options-heading'],
      ],
    ];

    $form['scan_options']['scan_contrib'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Scan contributed modules'),
      '#default_value' => TRUE,
      '#prefix' => '<div class="scan-option">',
      '#suffix' => '</div>',
    ];

    $form['scan_options']['scan_custom'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Scan custom modules'),
      '#default_value' => TRUE,
      '#prefix' => '<div class="scan-option">',
      '#suffix' => '</div>',
    ];

    // Get module counts to show alongside checkboxes.
    $contrib_modules = $this->moduleDiscovery->getModulesByType('contrib');
    $custom_modules = $this->moduleDiscovery->getModulesByType('custom');

    $contrib_count = count($contrib_modules);
    $custom_count = count($custom_modules);

    if ($contrib_count > 0) {
      $form['scan_options']['scan_contrib']['#description'] = $this->formatPlural(
        $contrib_count,
        '1 contributed module installed',
        '@count contributed modules installed'
      );
    }

    if ($custom_count > 0) {
      $form['scan_options']['scan_custom']['#description'] = $this->formatPlural(
        $custom_count,
        '1 custom module installed',
        '@count custom modules installed'
      );
    }

    // Submit button.
    $form['actions'] = [
      '#type' => 'actions',
      '#attributes' => [
        'class' => ['scan-button'],
      ],
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Run Compatibility Scan'),
      '#button_type' => 'primary',
    ];

    // Display results if available.
    $scan_results = $this->scannerService->getScanResults();
    if (!empty($scan_results['modules'])) {
      $form['results'] = [
        '#theme' => 'php_compatibility_scanner_report',
        '#current_version' => $scan_results['current_version'] ?? $current_version,
        '#target_version' => $scan_results['target_version'] ?? $target_version,
        '#modules' => $scan_results['modules'],
        '#attached' => [
          'library' => [
            'php_compatibility_scanner/php_compatibility_scanner.styles',
            'php_compatibility_scanner/php_compatibility_scanner.modal',
            'core/drupal.dialog.ajax',
          ],
        ],
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Ensure at least one module type is selected.
    if (!$form_state->getValue('scan_contrib') && !$form_state->getValue('scan_custom')) {
      $form_state->setError($form['scan_options'], $this->t('Please select at least one module type to scan.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Determine which modules to scan.
    $modules_to_scan = [];

    if ($form_state->getValue('scan_contrib')) {
      $contrib_modules = $this->moduleDiscovery->getModulesByType('contrib');
      $modules_to_scan = array_merge($modules_to_scan, array_keys($contrib_modules));
    }

    if ($form_state->getValue('scan_custom')) {
      $custom_modules = $this->moduleDiscovery->getModulesByType('custom');
      $modules_to_scan = array_merge($modules_to_scan, array_keys($custom_modules));
    }

    // Get target PHP version.
    $target_version = $this->scannerService->getNextPhpVersion();

    // Create and set the batch.
    $batch = \Drupal\php_compatibility_scanner\Batch\ScanBatch::createBatch($modules_to_scan, $target_version);
    batch_set($batch);
  }

}