<?php

namespace Drupal\php_compatibility_scanner\Service;

use Drupal\Core\Extension\ModuleExtensionList;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Cache\CacheBackendInterface;

/**
 * Service for discovering and categorizing modules.
 */
class ModuleDiscoveryService {

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The module extension list.
   *
   * @var \Drupal\Core\Extension\ModuleExtensionList
   */
  protected $moduleExtensionList;

  /**
   * The cache backend.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * Constructs a new ModuleDiscoveryService object.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\Core\Extension\ModuleExtensionList $module_extension_list
   *   The module extension list.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   The cache backend.
   */
  public function __construct(
    ModuleHandlerInterface $module_handler,
    ModuleExtensionList $module_extension_list,
    CacheBackendInterface $cache
  ) {
    $this->moduleHandler = $module_handler;
    $this->moduleExtensionList = $module_extension_list;
    $this->cache = $cache;
  }

  /**
   * Gets all installed modules.
   *
   * @return array
   *   An array of module information.
   */
  public function getAllModules() {
    $cid = 'php_compatibility_scanner:all_modules';
    $cached = $this->cache->get($cid);

    if ($cached) {
      return $cached->data;
    }

    $modules = [];
    $module_list = $this->moduleHandler->getModuleList();

    foreach ($module_list as $name => $extension) {
      $modules[$name] = [
        'name' => $name,
        'type' => $this->getModuleType($name),
        'path' => $extension->getPath(),
        'info' => $this->moduleExtensionList->getExtensionInfo($name),
        'status' => TRUE,
      ];
    }

    // Cache for 1 hour or until module list changes.
    $this->cache->set($cid, $modules, time() + 3600, ['module_list']);

    return $modules;
  }

  /**
   * Gets modules by type (core, contrib, custom).
   *
   * @param string $type
   *   The type of modules to get (core, contrib, custom).
   *
   * @return array
   *   An array of modules of the requested type.
   */
  public function getModulesByType($type) {
    $modules = $this->getAllModules();

    return array_filter($modules, function ($module) use ($type) {
      return $module['type'] === $type;
    });
  }

  /**
   * Gets the type of a module (core, contrib, custom).
   *
   * @param string $name
   *   The name of the module.
   *
   * @return string
   *   The type of the module (core, contrib, custom).
   */
  public function getModuleType($name) {
    // Core modules are in core directory.
    if ($this->isCore($name)) {
      return 'core';
    }

    // Determine if contrib or custom based on path.
    $extension = $this->moduleHandler->getModule($name);
    $path = $extension->getPath();

    // Check for common contrib directories.
    if (strpos($path, 'modules/contrib') !== FALSE ||
        strpos($path, 'sites/all/modules/contrib') !== FALSE) {
      return 'contrib';
    }

    // Check for composer vendor directory.
    if (strpos($path, 'vendor/') !== FALSE && strpos($path, 'drupal/') !== FALSE) {
      return 'contrib';
    }

    // Assume custom otherwise.
    return 'custom';
  }

  /**
   * Determines if a module is a core module.
   *
   * @param string $name
   *   The name of the module.
   *
   * @return bool
   *   TRUE if the module is a core module, FALSE otherwise.
   */
  public function isCore($name) {
    $extension = $this->moduleHandler->getModule($name);
    $path = $extension->getPath();

    // Core modules are in core directory or have a core component.
    return strpos($path, 'core/modules') === 0 ||
           strpos($path, 'core/profiles') === 0 ||
           strpos($path, 'modules/system/tests/modules') !== FALSE ||
           isset($extension->info['package']) && $extension->info['package'] === 'Core';
  }

  /**
   * Gets the path to a module.
   *
   * @param string $name
   *   The name of the module.
   *
   * @return string
   *   The path to the module.
   */
  public function getModulePath($name) {
    $extension = $this->moduleHandler->getModule($name);
    return $extension->getPath();
  }

}