<?php

namespace Drupal\php_compatibility_scanner\Service;

use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\State\StateInterface;

/**
 * Service for scanning PHP compatibility.
 */
class ScannerService {

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The module discovery service.
   *
   * @var \Drupal\php_compatibility_scanner\Service\ModuleDiscoveryService
   */
  protected $moduleDiscovery;

  /**
   * Constructs a new ScannerService.
   */
  public function __construct(FileSystemInterface $file_system, StateInterface $state, ModuleDiscoveryService $module_discovery) {
    $this->fileSystem = $file_system;
    $this->state = $state;
    $this->moduleDiscovery = $module_discovery;
  }

  /**
   * Get the current PHP version.
   *
   * @return string
   *   The PHP version in major.minor format.
   */
  public function getCurrentPhpVersion() {
    $version = phpversion();
    // Extract major.minor version
    preg_match('/^(\d+\.\d+)/', $version, $matches);
    return $matches[1] ?? $version;
  }

  /**
   * Get available PHP versions for scanning.
   *
   * @return array
   *   Array of PHP versions.
   */
  public function getAvailablePhpVersions() {
    $current = $this->getCurrentPhpVersion();
    $versions = [];

    // Define known PHP versions
    $all_versions = ['7.4', '8.0', '8.1', '8.2', '8.3', '8.4'];

    // Only show versions newer than current
    foreach ($all_versions as $version) {
      if (version_compare($version, $current, '>')) {
        $versions[$version] = 'PHP ' . $version;
      }
    }

    return $versions;
  }

  /**
   * Get the next PHP version.
   *
   * @return string
   *   The next PHP version.
   */
  public function getNextPhpVersion() {
    $available = $this->getAvailablePhpVersions();
    return !empty($available) ? key($available) : $this->getCurrentPhpVersion();
  }

  /**
   * Verify PHPCompatibility is properly installed and configured.
   *
   * @return array
   *   Array with 'success' boolean and 'error' message if failed.
   */
  public function verifyPhpCompatibility() {
    $phpcs_path = $this->findPhpcs();
    if (!$phpcs_path) {
      return [
        'success' => FALSE,
        'error' => 'PHP_CodeSniffer not found. Install with: composer require squizlabs/php_codesniffer phpcompatibility/php-compatibility:dev-develop',
      ];
    }

    // Check if PHPCompatibility standard is available
    $command = sprintf('%s -i', escapeshellarg($phpcs_path));
    $output = [];
    $return_var = 0;
    exec($command, $output, $return_var);

    $standards_output = implode(' ', $output);
    if (strpos($standards_output, 'PHPCompatibility') === FALSE) {
      return [
        'success' => FALSE,
        'error' => 'PHPCompatibility standard not found. Install with: composer require phpcompatibility/php-compatibility:dev-develop dealerdirect/phpcodesniffer-composer-installer',
      ];
    }

    return ['success' => TRUE];
  }

  /**
   * Scan a module for PHP compatibility issues.
   *
   * @param string $module_name
   *   The module name.
   * @param string $target_version
   *   The target PHP version.
   *
   * @return array
   *   Array with 'success' boolean and 'issues' array.
   */
  public function scanModule($module_name, $target_version) {
    // First verify PHPCompatibility is available
    $verification = $this->verifyPhpCompatibility();
    if (!$verification['success']) {
      return [
        'success' => FALSE,
        'error' => $verification['error'],
        'issues' => [],
      ];
    }

    $module_path = $this->moduleDiscovery->getModulePath($module_name);
    if (!$module_path) {
      return [
        'success' => FALSE,
        'error' => 'Module path not found',
        'issues' => [],
      ];
    }

    // Run PHPCompatibility scan
    $phpcs_issues = $this->runPhpcsCompatibilityScan($module_name, $module_path, $target_version);

    // Add custom compatibility checks for issues PHPCompatibility misses
    $custom_issues = $this->runCustomCompatibilityChecks($module_path, $target_version);

    // Merge all issues
    $all_issues = array_merge($phpcs_issues, $custom_issues);

    return [
      'success' => TRUE,
      'issues' => $all_issues,
    ];
  }

  /**
   * Run PHPCompatibility scan using PHPCS.
   */
  protected function runPhpcsCompatibilityScan($module_name, $module_path, $target_version) {
    $phpcs_path = $this->findPhpcs();
    $real_path = $this->fileSystem->realpath($module_path);

    // Use correct testVersion format with dash for ranges
    $test_version = $target_version . '-';

    // Build command
    $command = sprintf(
      '%s --standard=PHPCompatibility --runtime-set testVersion %s --report=json --extensions=php,module,inc,install,theme --ignore="*/vendor/*,*/node_modules/*" %s 2>&1',
      escapeshellarg($phpcs_path),
      escapeshellarg($test_version),
      escapeshellarg($real_path)
    );

    // Execute command
    $output = [];
    $return_var = 0;
    exec($command, $output, $return_var);

    // Parse JSON output
    $json_output = implode("\n", $output);
    $data = json_decode($json_output, TRUE);
    if (json_last_error() !== JSON_ERROR_NONE) {
      return [];
    }

    // Extract issues
    $issues = [];
    if (isset($data['files']) && is_array($data['files'])) {
      foreach ($data['files'] as $file_path => $file_data) {
        if (isset($file_data['messages']) && is_array($file_data['messages'])) {
          foreach ($file_data['messages'] as $message) {
            $issues[] = [
              'file' => str_replace($real_path . '/', '', $file_path),
              'line' => $message['line'] ?? 0,
              'message' => $message['message'] ?? 'Unknown issue',
              'severity' => isset($message['type']) ? strtolower($message['type']) : 'warning',
            ];
          }
        }
      }
    }

    return $issues;
  }

  /**
   * Run custom compatibility checks for issues PHPCompatibility misses.
   */
  protected function runCustomCompatibilityChecks($module_path, $target_version) {
    $issues = [];

    // Check for dynamic properties (deprecated in PHP 8.2)
    if (version_compare($target_version, '8.2', '>=')) {
      $dynamic_property_issues = $this->checkForDynamicProperties($module_path);
      $issues = array_merge($issues, $dynamic_property_issues);
    }

    // Check for other PHP 8.1+ issues
    if (version_compare($target_version, '8.1', '>=')) {
      $other_issues = $this->checkForPhp81Issues($module_path);
      $issues = array_merge($issues, $other_issues);
    }

    return $issues;
  }

  /**
   * Check for dynamic property creation (deprecated in PHP 8.2).
   */
  protected function checkForDynamicProperties($module_path) {
    $issues = [];
    $real_path = $this->fileSystem->realpath($module_path);

    // Find all PHP files
    $iterator = new \RecursiveIteratorIterator(
      new \RecursiveDirectoryIterator($real_path, \RecursiveDirectoryIterator::SKIP_DOTS)
    );

    foreach ($iterator as $file) {
      if ($file->getExtension() === 'php' || $file->getExtension() === 'module' || $file->getExtension() === 'inc') {
        $issues = array_merge($issues, $this->scanFileForDynamicProperties($file));
      }
    }

    return $issues;
  }

  /**
   * Scan a single file for dynamic property creation.
   */
  protected function scanFileForDynamicProperties($file) {
    $issues = [];
    $content = file_get_contents($file->getPathname());
    $lines = explode("\n", $content);

    // Get class and property declarations to avoid false positives
    $declared_properties = $this->extractDeclaredProperties($content);

    foreach ($lines as $line_number => $line) {
      // Skip comments and strings to reduce false positives
      $clean_line = preg_replace('/\/\*.*?\*\/|\/\/.*$|["\'].*?["\']/', '', $line);

      // Look for patterns like $this->property_name = value
      if (preg_match('/\$this\s*->\s*([a-zA-Z_][a-zA-Z0-9_]*)\s*=/', $clean_line, $matches)) {
        $property_name = $matches[1];

        // Skip if this is a declared property
        if (!in_array($property_name, $declared_properties)) {
          // Additional checks to reduce false positives
          if (!$this->isLikelyFalsePositive($line, $property_name)) {
            $issues[] = [
              'file' => basename($file->getPathname()),
              'line' => $line_number + 1,
              'message' => "Dynamic property creation (\$this->{$property_name}) deprecated in PHP 8.2. Consider declaring the property in the class or using a different approach.",
              'severity' => 'warning',
            ];
          }
        }
      }
    }

    return $issues;
  }

  /**
   * Extract declared properties from class content.
   */
  protected function extractDeclaredProperties($content) {
    $properties = [];

    // Match property declarations
    if (preg_match_all('/(?:public|protected|private)\s+(?:static\s+)?\$([a-zA-Z_][a-zA-Z0-9_]*)/m', $content, $matches)) {
      $properties = array_merge($properties, $matches[1]);
    }

    return $properties;
  }

  /**
   * Check if this is likely a false positive for dynamic property detection.
   */
  protected function isLikelyFalsePositive($line, $property_name) {
    $escaped_property_name = preg_quote($property_name);
    $false_positive_patterns = [
      // Comments
      '/^\s*\/\//',
      '/^\s*\*/',
      // String concatenation or array access
      '/\$this->' . $escaped_property_name . '\[/',
      // Method calls that look like property assignment
      '/\$this->' . $escaped_property_name . '\s*\(/',
    ];

    foreach ($false_positive_patterns as $pattern) {
      if (preg_match($pattern, $line)) {
        return TRUE;
      }
    }

    return FALSE;
  }

  /**
   * Check for other PHP 8.1+ compatibility issues.
   */
  protected function checkForPhp81Issues($module_path) {
    $issues = [];

    // Add checks for other common PHP 8.1+ issues that PHPCompatibility might miss
    // This can be expanded as needed

    return $issues;
  }

  /**
   * Find the phpcs executable with expanded search paths.
   *
   * @return string|false
   *   Path to phpcs or FALSE if not found.
   */
  protected function findPhpcs() {
    $possible_paths = [
      // Drupal site vendor
      DRUPAL_ROOT . '/../vendor/bin/phpcs',
      DRUPAL_ROOT . '/vendor/bin/phpcs',

      // Global composer
      $_SERVER['HOME'] . '/.composer/vendor/bin/phpcs',
      $_SERVER['HOME'] . '/.config/composer/vendor/bin/phpcs',

      // Project root variations
      DRUPAL_ROOT . '/../../vendor/bin/phpcs',
      DRUPAL_ROOT . '/../../../vendor/bin/phpcs',

      // System PATH
      'phpcs', // Will work if phpcs is in system PATH
    ];

    foreach ($possible_paths as $path) {
      if ($path === 'phpcs') {
        // Check if phpcs is available in system PATH
        $test_output = [];
        $return_var = 0;
        exec('which phpcs 2>/dev/null', $test_output, $return_var);
        if ($return_var === 0 && !empty($test_output[0])) {
          return trim($test_output[0]);
        }
        continue;
      }

      if (file_exists($path) && is_executable($path)) {
        return $path;
      }
    }

    return FALSE;
  }

  /**
   * Save scan results to state.
   *
   * @param array $results
   *   The scan results.
   */
  public function saveScanResults(array $results) {
    $this->state->set('php_compatibility_scanner.last_scan', time());
    $this->state->set('php_compatibility_scanner.scan_results', $results);
  }

  /**
   * Get saved scan results.
   *
   * @return array
   *   The scan results.
   */
  public function getScanResults() {
    return $this->state->get('php_compatibility_scanner.scan_results', []);
  }

  /**
   * Get last scan time.
   *
   * @return int|null
   *   Unix timestamp or NULL.
   */
  public function getLastScanTime() {
    return $this->state->get('php_compatibility_scanner.last_scan');
  }
}