<?php

namespace Drupal\Tests\php_compatibility_scanner\Unit\Service;

use Drupal\Tests\UnitTestCase;
use Drupal\php_compatibility_scanner\Service\ScannerService;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\State\StateInterface;
use Drupal\php_compatibility_scanner\Service\ModuleDiscoveryService;

/**
 * Tests for the ScannerService.
 *
 * @group php_compatibility_scanner
 */
class ScannerServiceTest extends UnitTestCase {

  /**
   * The scanner service under test.
   *
   * @var \Drupal\php_compatibility_scanner\Service\ScannerService
   */
  protected $scannerService;

  /**
   * Mock file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $fileSystem;

  /**
   * Mock state service.
   *
   * @var \Drupal\Core\State\StateInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $state;

  /**
   * Mock module discovery service.
   *
   * @var \Drupal\php_compatibility_scanner\Service\ModuleDiscoveryService|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $moduleDiscovery;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->fileSystem = $this->createMock(FileSystemInterface::class);
    $this->state = $this->createMock(StateInterface::class);
    $this->moduleDiscovery = $this->createMock(ModuleDiscoveryService::class);

    $this->scannerService = new ScannerService(
      $this->fileSystem,
      $this->state,
      $this->moduleDiscovery
    );
  }

  /**
   * Test getCurrentPhpVersion method returns proper format.
   */
  public function testGetCurrentPhpVersion() {
    $version = $this->scannerService->getCurrentPhpVersion();
    $this->assertMatchesRegularExpression('/^\d+\.\d+$/', $version);
    $this->assertIsString($version);
  }

  /**
   * Test getAvailablePhpVersions returns array.
   */
  public function testGetAvailablePhpVersions() {
    $versions = $this->scannerService->getAvailablePhpVersions();
    $this->assertIsArray($versions);
    
    // Each key should be a version number, each value should be a label
    foreach ($versions as $version => $label) {
      $this->assertMatchesRegularExpression('/^\d+\.\d+$/', $version);
      $this->assertStringContainsString('PHP', $label);
    }
  }

  /**
   * Test getNextPhpVersion returns valid version.
   */
  public function testGetNextPhpVersion() {
    $nextVersion = $this->scannerService->getNextPhpVersion();
    $this->assertMatchesRegularExpression('/^\d+\.\d+$/', $nextVersion);
  }

  /**
   * Test state storage methods.
   */
  public function testStateMethods() {
    $testResults = [
      'current_version' => '8.1',
      'target_version' => '8.2',
      'modules' => ['test_module' => ['issues' => []]],
    ];

    // The saveScanResults method calls set() twice, in this order:
    // 1. For last_scan timestamp
    // 2. For scan_results data
    $this->state->expects($this->exactly(2))
      ->method('set')
      ->withConsecutive(
        ['php_compatibility_scanner.last_scan', $this->isType('integer')],
        ['php_compatibility_scanner.scan_results', $testResults]
      );

    $this->scannerService->saveScanResults($testResults);
  }

}
